@@ -1,16 +1,37 @@
+/*
+ * Licensed to Elasticsearch B.V. under one or more contributor
+ * license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright
+ * ownership. Elasticsearch B.V. licenses this file to you under
+ * the Apache License, Version 2.0 (the "License"); you may
+ * not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *    http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
+ */
+
 import $ from 'jquery';
 import _ from 'lodash';
-import AggConfigResult from 'ui/vis/agg_config_result';
-import { FilterBarClickHandlerProvider } from 'ui/filter_bar/filter_bar_click_handler';
-import { uiModules } from 'ui/modules';
+import AggConfigResult from '../vis/agg_config_result';
+import { uiModules } from '../modules';
 import tableCellFilterHtml from './partials/table_cell_filter.html';
+import { isNumeric } from '../utils/numeric';
+import { VisFiltersProvider } from '../vis/vis_filters';
+
 const module = uiModules.get('kibana');
 
-module.directive('kbnRows', function ($compile, $rootScope, getAppState, Private) {
-  const filterBarClickHandler = Private(FilterBarClickHandlerProvider);
+module.directive('kbnRows', function ($compile, Private) {
   return {
     restrict: 'A',
     link: function ($scope, $el, attr) {
+      const visFilter = Private(VisFiltersProvider);
       function addCell($tr, contents) {
         function createCell() {
           return $(document.createElement('td'));
@@ -18,19 +39,17 @@ module.directive('kbnRows', function ($compile, $rootScope, getAppState, Private
 
         function createFilterableCell(aggConfigResult) {
           const $template = $(tableCellFilterHtml);
-          $template.addClass('cell-hover');
+          $template.addClass('kbnTableCellFilter__hover');
 
           const scope = $scope.$new();
 
-          const $state = getAppState();
-          const addFilter = filterBarClickHandler($state);
           scope.onFilterClick = (event, negate) => {
             // Don't add filter if a link was clicked.
             if ($(event.target).is('a')) {
               return;
             }
 
-            addFilter({ point: { aggConfigResult: aggConfigResult }, negate });
+            visFilter.filter({ datum: { aggConfigResult: aggConfigResult }, negate });
           };
 
           return $compile($template)(scope);
@@ -60,7 +79,7 @@ module.directive('kbnRows', function ($compile, $rootScope, getAppState, Private
 
           // TODO: It would be better to actually check the type of the field, but we don't have
           // access to it here. This may become a problem with the switch to BigNumber
-          if (_.isNumeric(contents)) {
+          if (isNumeric(contents)) {
             $cell.addClass('numeric-value');
           }
         }
