@@ -1,38 +1,57 @@
+/*
+ * Licensed to Elasticsearch B.V. under one or more contributor
+ * license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright
+ * ownership. Elasticsearch B.V. licenses this file to you under
+ * the Apache License, Version 2.0 (the "License"); you may
+ * not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *    http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
+ */
+
 import _ from 'lodash';
-import template from 'ui/filter_bar/filter_bar.html';
-import 'ui/directives/json_input';
+import template from './filter_bar.html';
+import '../directives/json_input';
 import '../filter_editor';
 import './filter_pill/filter_pill';
-import { filterAppliedAndUnwrap } from 'ui/filter_bar/lib/filter_applied_and_unwrap';
-import { FilterBarLibMapAndFlattenFiltersProvider } from 'ui/filter_bar/lib/map_and_flatten_filters';
-import { FilterBarLibMapFlattenAndWrapFiltersProvider } from 'ui/filter_bar/lib/map_flatten_and_wrap_filters';
-import { FilterBarLibExtractTimeFilterProvider } from 'ui/filter_bar/lib/extract_time_filter';
-import { FilterBarLibFilterOutTimeBasedFilterProvider } from 'ui/filter_bar/lib/filter_out_time_based_filter';
-import { FilterBarLibChangeTimeFilterProvider } from 'ui/filter_bar/lib/change_time_filter';
-import { FilterBarQueryFilterProvider } from 'ui/filter_bar/query_filter';
+import { filterAppliedAndUnwrap } from './lib/filter_applied_and_unwrap';
+import { FilterBarLibMapAndFlattenFiltersProvider } from './lib/map_and_flatten_filters';
+import { FilterBarLibMapFlattenAndWrapFiltersProvider } from './lib/map_flatten_and_wrap_filters';
+import { FilterBarLibExtractTimeFilterProvider } from './lib/extract_time_filter';
+import { FilterBarLibFilterOutTimeBasedFilterProvider } from './lib/filter_out_time_based_filter';
+import { changeTimeFilter } from './lib/change_time_filter';
+import { FilterBarQueryFilterProvider } from './query_filter';
 import { compareFilters } from './lib/compare_filters';
-import { uiModules } from 'ui/modules';
+import { uiModules } from '../modules';
 
 export { disableFilter, enableFilter, toggleFilterDisabled } from './lib/disable_filter';
 
 
 const module = uiModules.get('kibana');
 
-module.directive('filterBar', function (Private, Promise, getAppState) {
+module.directive('filterBar', function (Private, Promise, getAppState, i18n) {
   const mapAndFlattenFilters = Private(FilterBarLibMapAndFlattenFiltersProvider);
   const mapFlattenAndWrapFilters = Private(FilterBarLibMapFlattenAndWrapFiltersProvider);
   const extractTimeFilter = Private(FilterBarLibExtractTimeFilterProvider);
   const filterOutTimeBasedFilter = Private(FilterBarLibFilterOutTimeBasedFilterProvider);
-  const changeTimeFilter = Private(FilterBarLibChangeTimeFilterProvider);
   const queryFilter = Private(FilterBarQueryFilterProvider);
 
   return {
     template,
     restrict: 'E',
     scope: {
-      indexPatterns: '='
+      indexPatterns: '=',
+      tooltipContent: '=',
     },
-    link: function ($scope) {
+    link: function ($scope, $elem) {
       // bind query filter actions to the scope
       [
         'addFilters',
@@ -50,8 +69,35 @@ module.directive('filterBar', function (Private, Promise, getAppState) {
 
       $scope.state = getAppState();
 
-      $scope.showAddFilterButton = () => {
-        return _.compact($scope.indexPatterns).length > 0;
+      $scope.showCollapseLink = () => {
+        const pill = $elem.find('filter-pill');
+        return pill[pill.length - 1].offsetTop > 10;
+      };
+
+      const collapseFilterTooltip = i18n('common.ui.filterBar.collapseFilterTooltip', {
+        defaultMessage: 'Collapse filter bar \n to show less'
+      });
+      const expandFilterTooltip = i18n('common.ui.filterBar.expandFilterTooltip', { defaultMessage: 'Expand filter bar \n to show more' });
+
+      $scope.filterNavToggle = {
+        isOpen: true,
+        tooltipContent: collapseFilterTooltip
+      };
+
+      $scope.toggleFilterShown = () => {
+        const collapser = $elem.find('.filter-nav-link__collapser');
+        const filterPanelPill = $elem.find('.filter-panel__pill');
+        if ($scope.filterNavToggle.isOpen) {
+          $scope.filterNavToggle.tooltipContent = expandFilterTooltip;
+          collapser.attr('aria-expanded', 'false');
+          filterPanelPill.attr('style', 'width: calc(100% - 80px)');
+        } else {
+          $scope.filterNavToggle.tooltipContent = collapseFilterTooltip;
+          collapser.attr('aria-expanded', 'true');
+          filterPanelPill.attr('style', 'width: auto');
+        }
+
+        $scope.filterNavToggle.isOpen = !$scope.filterNavToggle.isOpen;
       };
 
       $scope.applyFilters = function (filters) {
@@ -112,29 +158,29 @@ module.directive('filterBar', function (Private, Promise, getAppState) {
         // users to decide what they want to apply.
         if (filters.length > 1) {
           return mapFlattenAndWrapFilters(filters)
-          .then(function (results) {
-            extractTimeFilter(results).then(function (filter) {
-              $scope.changeTimeFilter = filter;
+            .then(function (results) {
+              extractTimeFilter(results).then(function (filter) {
+                $scope.changeTimeFilter = filter;
+              });
+              return results;
+            })
+            .then(filterOutTimeBasedFilter)
+            .then(function (results) {
+              $scope.newFilters = results;
             });
-            return results;
-          })
-          .then(filterOutTimeBasedFilter)
-          .then(function (results) {
-            $scope.newFilters = results;
-          });
         }
 
         // Just add single filters to the state.
         if (filters.length === 1) {
           Promise.resolve(filters).then(function (filters) {
             extractTimeFilter(filters)
-            .then(function (timeFilter) {
-              if (timeFilter) changeTimeFilter(timeFilter);
-            });
+              .then(function (timeFilter) {
+                if (timeFilter) changeTimeFilter(timeFilter);
+              });
             return filters;
           })
-          .then(filterOutTimeBasedFilter)
-          .then(addAndInvertFilters);
+            .then(filterOutTimeBasedFilter)
+            .then(addAndInvertFilters);
         }
       });
 
