@@ -1,5 +1,25 @@
-import html from 'ui/timepicker/timepicker.html';
+/*
+ * Licensed to Elasticsearch B.V. under one or more contributor
+ * license agreements. See the NOTICE file distributed with
+ * this work for additional information regarding copyright
+ * ownership. Elasticsearch B.V. licenses this file to you under
+ * the Apache License, Version 2.0 (the "License"); you may
+ * not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *    http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
+ */
+
+import html from './timepicker.html';
 import './quick_panel';
+import './recent_panel';
 import './relative_panel';
 import './absolute_panel';
 import _ from 'lodash';
@@ -7,21 +27,19 @@ import { relativeOptions } from './relative_options';
 import { parseRelativeParts } from './parse_relative_parts';
 import dateMath from '@elastic/datemath';
 import moment from 'moment';
-import { Notifier } from 'ui/notify/notifier';
-import 'ui/timepicker/timepicker.less';
-import 'ui/directives/input_datetime';
-import 'ui/directives/inequality';
-import 'ui/timepicker/quick_ranges';
-import 'ui/timepicker/refresh_intervals';
-import 'ui/timepicker/time_units';
-import 'ui/timepicker/kbn_global_timepicker';
-import { uiModules } from 'ui/modules';
+import './timepicker.less';
+import '../directives/input_datetime';
+import '../directives/inequality';
+import './refresh_intervals';
+import './kbn_global_timepicker';
+import { uiModules } from '../modules';
+import { TIME_MODES } from './modes';
+import { timeUnits } from './time_units';
+import { prettyInterval } from './pretty_interval';
+import { i18n } from '@kbn/i18n';
 const module = uiModules.get('ui/timepicker');
-const notify = new Notifier({
-  location: 'timepicker',
-});
 
-module.directive('kbnTimepicker', function (timeUnits, refreshIntervals) {
+module.directive('kbnTimepicker', function (refreshIntervals) {
   return {
     restrict: 'E',
     scope: {
@@ -36,10 +54,10 @@ module.directive('kbnTimepicker', function (timeUnits, refreshIntervals) {
     template: html,
     controller: function ($scope) {
       $scope.format = 'MMMM Do YYYY, HH:mm:ss.SSS';
-      $scope.modes = ['quick', 'relative', 'absolute'];
+      $scope.modes = Object.values(TIME_MODES);
       $scope.activeTab = $scope.activeTab || 'filter';
 
-      if (_.isUndefined($scope.mode)) $scope.mode = 'quick';
+      if (_.isUndefined($scope.mode)) $scope.mode = TIME_MODES.QUICK;
 
       $scope.refreshLists = _(refreshIntervals).groupBy('section').values().value();
 
@@ -68,13 +86,13 @@ module.directive('kbnTimepicker', function (timeUnits, refreshIntervals) {
       $scope.relativeOptions = relativeOptions;
 
       $scope.$watch('from', function (date) {
-        if (moment.isMoment(date) && $scope.mode === 'absolute') {
+        if (moment.isMoment(date) && $scope.mode === TIME_MODES.ABSOLUTE) {
           $scope.absolute.from = date;
         }
       });
 
       $scope.$watch('to', function (date) {
-        if (moment.isMoment(date) && $scope.mode === 'absolute') {
+        if (moment.isMoment(date) && $scope.mode === TIME_MODES.ABSOLUTE) {
           $scope.absolute.to = date;
         }
       });
@@ -82,14 +100,22 @@ module.directive('kbnTimepicker', function (timeUnits, refreshIntervals) {
       // If we always return a new object from the getters below (pickFromDate and pickToDate) we'll create an
       // infinite digest loop, so we maintain these copies to return instead.
       $scope.$watch('absolute.from', function (newDate) {
+        if (!newDate) {
+          return;
+        }
+
         _.set($scope, 'browserAbsolute.from', new Date(newDate.year(), newDate.month(), newDate.date()));
       });
 
       $scope.$watch('absolute.to', function (newDate) {
+        if (!newDate) {
+          return;
+        }
+
         _.set($scope, 'browserAbsolute.to', new Date(newDate.year(), newDate.month(), newDate.date()));
       });
 
-      // The datepicker directive uses native Javascript Dates, ignoring moment's default timezone. This causes
+      // The datepicker directive uses native JavaScript Dates, ignoring moment's default timezone. This causes
       // the datepicker and the text input above it to get out of sync if the user changed the `dateFormat:tz` config
       // in advanced settings. The text input will show the date in the user selected timezone, the datepicker will
       // show the date in the local browser timezone. Since we really just want a day, month, year from the datepicker
@@ -126,16 +152,18 @@ module.directive('kbnTimepicker', function (timeUnits, refreshIntervals) {
 
       $scope.setMode = function (thisMode) {
         switch (thisMode) {
-          case 'quick':
+          case TIME_MODES.QUICK:
             break;
-          case 'relative':
+          case TIME_MODES.RECENT:
+            break;
+          case TIME_MODES.RELATIVE:
             $scope.relative = parseRelativeParts($scope.from, $scope.to);
             $scope.formatRelative('from');
             $scope.formatRelative('to');
             break;
-          case 'absolute':
+          case TIME_MODES.ABSOLUTE:
             $scope.absolute.from = dateMath.parse($scope.from || moment().subtract(15, 'minutes'));
-            $scope.absolute.to = dateMath.parse($scope.to || moment(), true);
+            $scope.absolute.to = dateMath.parse($scope.to || moment(), { roundUp: true });
             break;
         }
 
@@ -143,7 +171,7 @@ module.directive('kbnTimepicker', function (timeUnits, refreshIntervals) {
       };
 
       $scope.setQuick = function (from, to) {
-        $scope.onFilterSelect({ from, to });
+        $scope.onFilterSelect({ from, to, mode: TIME_MODES.QUICK });
       };
 
       $scope.setToNow = function (key) {
@@ -159,7 +187,7 @@ module.directive('kbnTimepicker', function (timeUnits, refreshIntervals) {
       $scope.checkRelative = function () {
         if ($scope.relative.from.count != null && $scope.relative.to.count != null) {
           const from = dateMath.parse(getRelativeString('from'));
-          const to = dateMath.parse(getRelativeString('to'), true);
+          const to = dateMath.parse(getRelativeString('to'), { roundUp: true });
           if (to && from) return to.isBefore(from);
           return true;
         }
@@ -167,10 +195,10 @@ module.directive('kbnTimepicker', function (timeUnits, refreshIntervals) {
 
       $scope.formatRelative = function (key) {
         const relativeString = getRelativeString(key);
-        const parsed = dateMath.parse(relativeString, key === 'to');
+        const parsed = dateMath.parse(relativeString, { roundUp: key === 'to' });
         let preview;
         if (relativeString === 'now') {
-          preview = 'Now';
+          preview = i18n.translate('common.ui.timepicker.now', { defaultMessage: 'Now' });
         } else {
           preview = parsed ? parsed.format($scope.format) : undefined;
         }
@@ -181,7 +209,8 @@ module.directive('kbnTimepicker', function (timeUnits, refreshIntervals) {
       $scope.applyRelative = function () {
         $scope.onFilterSelect({
           from: getRelativeString('from'),
-          to:  getRelativeString('to')
+          to: getRelativeString('to'),
+          mode: TIME_MODES.RELATIVE,
         });
       };
 
@@ -202,18 +231,25 @@ module.directive('kbnTimepicker', function (timeUnits, refreshIntervals) {
       $scope.applyAbsolute = function () {
         $scope.onFilterSelect({
           from: moment($scope.absolute.from),
-          to: moment($scope.absolute.to)
+          to: moment($scope.absolute.to),
+          mode: TIME_MODES.ABSOLUTE,
         });
       };
 
+      $scope.prettyInterval = function (interval) {
+        return prettyInterval(interval.value);
+      };
+
       $scope.setRefreshInterval = function (interval) {
         interval = _.clone(interval || {});
-        notify.log('before: ' + interval.pause);
         interval.pause = (interval.pause == null || interval.pause === false) ? false : true;
 
-        notify.log('after: ' + interval.pause);
-
-        $scope.onIntervalSelect({ interval });
+        $scope.onIntervalSelect({
+          interval: {
+            value: interval.value,
+            pause: interval.pause,
+          }
+        });
       };
 
       $scope.setMode($scope.mode);
